@@ -28,9 +28,17 @@
  * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
+ *
+ * End of legal section ......................................................
+ *
+ *  toos_dev_types.h - Defines an enum value for all mellanox chips in order to
+ *                     have a uniform IDs for all toos.
+ *
+ *  Version: $Id$
+ *
+ *  Author: Wasim Abo Moch (wasim@mellanox.co.il)
  */
 
-
 #ifndef TOOLS_DEV_TYPE_H
 #define TOOLS_DEV_TYPE_H
 
@@ -44,27 +52,45 @@ extern "C" {
 
 enum dm_dev_id
 {
+    DeviceUnknown = -1,       //Dummy Device - Marker for indicating error.
     DeviceStartMarker = 0,    // Dummy Device - Marker for first device
                               // to let user iterate from DeviceStartMarker to DeviceEndMarker
+                              // Note: Call dm_is_device_supported() to see if a device is supported by the lib.
 
-    DeviceInfiniScale = 0,
-    DeviceInfiniHost,
-    DeviceInfiniHostIIIEx,
-    DeviceInfiniHostIIIEx_MF,
-    DeviceInfiniScaleIII,
-    DeviceInfiniHostIIILx,
-    DeviceConnectX,
+    DeviceInfiniScale = 0,  // UnSupported
+    DeviceInfiniHost,       // UnSupported
+    DeviceInfiniHostIIIEx,  // UnSupported
+    DeviceInfiniHostIIIEx_MF,// UnSupported
+    DeviceInfiniScaleIII,   // UnSupported
+    DeviceInfiniHostIIILx,  // UnSupported
+    DeviceConnectX,         // UnSupported
     DeviceConnectX2,
     DeviceInfiniScaleIV,
-    DeviceBridgeX,
+    DeviceBridgeX,          // UnSupported
     DeviceSwitchX,
     DeviceConnectX3,
     DeviceConnectIB,
     DeviceConnectX3Pro,
     DeviceSwitchIB,
+    DeviceSpectrum,
+    DeviceQuantum,
     DeviceConnectX4,
+    DeviceConnectX4LX,
+    DeviceConnectX5,
+    DeviceBlueField,
+    DeviceFPGA,             // UnSupported
+    DeviceSwitchIB2,
+    DeviceFPGANewton,
+    DeviceCable,
+    DeviceCableQSFP,
+    DeviceCableQSFPaging,
+    DeviceCableSFP,
+    DeviceCableSFP51,
+    DeviceCableSFP51Paging,
+    DeviceSpectrum2,
+    DeviceDummy,
 
-    DeviceEndMarker           // Dummy Device - Marker for indicating error and end of devices
+    DeviceEndMarker           // Dummy Device - Marker for indicating end of devices when iterating
 };
 
 typedef enum dm_dev_id dm_dev_id_t;
@@ -84,11 +110,23 @@ int dm_get_device_id_offline(u_int32_t devid,
                              u_int32_t chip_rev,
                              dm_dev_id_t* ptr_dev_type);
 
+
+/**
+ * Returns 1 if device is supported and 0 otherwise (library dependant)
+ */
+int dm_is_device_supported(dm_dev_id_t type);
+
+
 /**
  * Returns the device name as a "const char*"
  */
 const char* dm_dev_type2str(dm_dev_id_t type);
 
+/**
+ * Returns the device id
+ */
+dm_dev_id_t dm_dev_str2type(const char* str);
+
 /**
  * A predicate returning if the device is an hca
  */
@@ -104,6 +142,11 @@ int dm_dev_is_switch(dm_dev_id_t type);
  */
 int dm_dev_is_bridge(dm_dev_id_t type);
 
+/**
+ * A predicate returning if the device is a Cable
+ */
+int dm_dev_is_cable(dm_dev_id_t type);
+
 /**
  * Returns the max num of ports or -1 on error
  */
@@ -116,10 +159,20 @@ u_int32_t dm_get_hw_dev_id(dm_dev_id_t type);
 
 /**
  * Returns the HW chip revision of the given device type or zero on failures,
- * This is useful to distinguish between hermon and hermon_b0.
+ * This is useful to distinguish between ConnectX2 and ConnectX.
  */
 u_int32_t dm_get_hw_rev_id(dm_dev_id_t type);
 
+/**
+ * A predicate returning if the device supports Function Per Port
+ */
+int dm_is_fpp_supported(dm_dev_id_t type);
+
+/*
+ * A predicate returning if the device is in LiveFish mode
+ */
+int dm_is_livefish_mode(mfile* mf);
+
 #ifdef __cplusplus
 }       /* end of 'extern "C"' */
 #endif
