@@ -108,6 +108,13 @@ SubCmdMetaData::SubCmdMetaData() {
     _sCmds.push_back(new SubCmd("", "rb", SC_Rb));
     _sCmds.push_back(new SubCmd("", "clear_semaphore", SC_Clear_Sem));
     _sCmds.push_back(new SubCmd("", "fi", SC_Fix_Img));
+    _sCmds.push_back(new SubCmd("cs", "checksum", SC_Check_Sum));
+    _sCmds.push_back(new SubCmd("ts", "timestamp", SC_Time_Stamp));
+    _sCmds.push_back(new SubCmd("ci", "cache_image", SC_Cache_Image));
+    _sCmds.push_back(new SubCmd("", "sign", SC_Sign));
+    _sCmds.push_back(new SubCmd("", "extract_fw_data", SC_Extract_4MB_Image));
+    _sCmds.push_back(new SubCmd("", "set_public_keys", SC_Set_Public_Keys));
+    _sCmds.push_back(new SubCmd("", "set_forbidden_versions", SC_Set_Forbidden_Versions));
 }
 
 SubCmdMetaData::~SubCmdMetaData() {
@@ -153,7 +160,6 @@ FlagMetaData::FlagMetaData() {
     _flags.push_back(new Flag("", "mac", 1));
     _flags.push_back(new Flag("", "macs", MACS));
     _flags.push_back(new Flag("", "uid", 1));
-    _flags.push_back(new Flag("", "uids", 29)); //TODO: update this with the correct constatnt from mlxfwops
     _flags.push_back(new Flag("", "blank_guids", 0));
     _flags.push_back(new Flag("", "clear_semaphore", 0));
     _flags.push_back(new Flag("h", "help", 0));
@@ -173,6 +179,7 @@ FlagMetaData::FlagMetaData() {
     _flags.push_back(new Flag("", "use_image_ps", 0));
     _flags.push_back(new Flag("", "use_image_guids", 0));
     _flags.push_back(new Flag("", "use_image_rom", 0));
+    _flags.push_back(new Flag("", "ignore_dev_data", 0));
     _flags.push_back(new Flag("", "dual_image", 0));
     _flags.push_back(new Flag("", "striped_image", 0));
     _flags.push_back(new Flag("", "banks", 1));
@@ -180,6 +187,13 @@ FlagMetaData::FlagMetaData() {
     _flags.push_back(new Flag("", "flash_params", 1)); //its actually 3 but separated by comma so we refer to them as one
     _flags.push_back(new Flag("v", "version", 0));
     _flags.push_back(new Flag("", "no_devid_check", 0));
+    _flags.push_back(new Flag("", "use_fw", 0));
+    _flags.push_back(new Flag("", "use_dev_img_info", 0));
+    _flags.push_back(new Flag("", "skip_ci_req", 0));
+    _flags.push_back(new Flag("", "use_dev_rom", 0));
+    _flags.push_back(new Flag("", "private_key", 1));
+    _flags.push_back(new Flag("", "key_uuid", 1));
+    _flags.push_back(new Flag("", "no_fw_ctrl", 0));
 }
 
 FlagMetaData::~FlagMetaData() {
@@ -283,7 +297,7 @@ bool verifyNumOfArgs(string name, string value) {
 bool strToNum(string str, u_int64_t& num, int base=0)
 {
     char *endp;
-    char* numStr = strcpy(new char[str.size()],str.c_str());
+    char* numStr = strcpy(new char[str.size() + 1],str.c_str());
     num = strtoul(numStr, &endp, base);
     if (*endp) {
         delete[] numStr;
@@ -301,7 +315,7 @@ bool getGUIDFromStr(string str, guid_t& guid, string prefixErr="")
     g = strtoull(str.c_str(), &endp, 16);
     if (*endp || (g == 0xffffffffffffffffULL && errno == ERANGE)) {
         if (prefixErr.size() == 0) {
-            printf("-E- Invalid GUID syntax (%s) %s \n", str.c_str(), errno ? strerror(errno) : "" );
+            printf("-E- Invalid Guid/Mac/Uid syntax (%s) %s \n", str.c_str(), errno ? strerror(errno) : "" );
         } else {
             printf("%s\n", prefixErr.c_str());
         }
@@ -331,54 +345,6 @@ guid_t GetBaseMac(guid_t baseGuid)
     return baseMac;
 }
 
-bool InitBxGuids(std::vector<guid_t>& user_guids, guid_t base_guid1) {
-
-    int base_index = 0;
-    guid_t base_mac, base_guid;
-    user_guids.resize(MAX_GUIDS);
-
-    for (int i = 0; i < BX_SLICES_NUM; i++) {
-        base_index = i * BX_SLICE_GUIDS;
-        base_guid = incrGuid(base_guid1, i * 8);
-        // Init Guids
-        int base_guids_index = base_index + BI_GUIDS;
-        for (int j = 0; j < BX_NP_GUIDS; j++) {
-            if (j == 0) {
-                // The node guid should be the same one on the two slices.
-                user_guids[base_guids_index + j] = incrGuid(base_guid1, j);
-            } else {
-                user_guids[base_guids_index + j] = incrGuid(base_guid, j);
-            }
-
-        }
-        // Init Macs
-        base_mac = GetBaseMac(base_guid);
-        int base_macs_index = base_index + BI_IMACS;
-        for (int j = 0; j < BX_MACS; j++) {
-            user_guids[base_macs_index + j] = incrGuid(base_mac, j);
-        }
-        // Init WWPNSs
-        int base_wwpn_index = base_index + BI_WWPNS;
-        int base_emac_index = base_index + BI_EMACS;
-
-        for (int j = 0; j < BX_WWPNS; j++) {
-            user_guids[base_wwpn_index + j].l =  user_guids[base_emac_index + j].l;
-            user_guids[base_wwpn_index + j].h = (user_guids[base_emac_index + j].h | (0x2000 << 16));
-        }
-
-        // Init WWNNS
-        int base_wwnn_index = base_index + BI_WWNNS;
-        base_emac_index = base_index + BI_EMACS;
-
-        user_guids[base_wwnn_index].l =  user_guids[base_emac_index].l;
-        user_guids[base_wwnn_index].h = (user_guids[base_emac_index].h | (0x2001 << 16));
-    }
-
-    // Init SysGuid
-    user_guids[BI_SYS_GUID] = incrGuid(base_guid1, 7);
-    return true;
-}
-
 bool parseFlashParams(string params, flash_params_t& fp)
 {
     // Step 1 split by ","
@@ -414,8 +380,12 @@ bool parseFlashParams(string params, flash_params_t& fp)
 #define FLASH_LIST_SZ 256
 
 void Flint::initCmdParser() {
-    AddDescription("flint is a FW (firmware) burning and flash memory operations tool for\n"
-            "Mellanox Infiniband HCAs, Ethernet NIC cards, and switch devices.");
+    bool isExternal = false;
+#ifdef EXTERNAL
+    isExternal = true;
+#endif
+    AddDescription("flint is a FW (firmware) burning and flash memory operations tool for Mellanox Infiniband HCAs,"
+                   "Ethernet NIC cards, and switch devices.");
 
     AddOptions("device",
                 'd',
@@ -442,48 +412,40 @@ void Flint::initCmdParser() {
     AddOptions("yes",
                'y',
                 "",
-                "Non interactive mode - assume answer\n"
-                "\"yes\" to all questions.\n"
+                "Non interactive mode - assume answer \"yes\" to all questions.\n"
                 "Commands affected: all");
 
     AddOptions("no",
                ' ',
                 "",
-                "Non interactive mode - assume answer\n"
-                "\"no\" to all questions.\n"
+                "Non interactive mode - assume answer \"no\" to all questions.\n"
                 "Commands affected: all");
 
     AddOptions("guid",
                ' ',
                 "<GUID>",
-                "GUID base value. 4 GUIDs\n"
-                "are automatically assigned to the following values:\n\n"
+                "GUID base value. 4 GUIDs are automatically assigned to the following values:\n\n"
                 "guid   -> node GUID\n"
                 "guid+1 -> port1\n"
                 "guid+2 -> port2\n"
                 "guid+3 -> system image GUID.\n\n"
-                "Note: port2 guid will be assigned even for a\n"
-                "single port HCA - The HCA ignores this value.\n\n"
+                "Note: port2 guid will be assigned even for a single port HCA - The HCA ignores this value.\n\n"
                 "Commands affected: burn, sg");
 
     AddOptions("guids",
                ' ',
                 "<GUIDS...>",
                 "4 GUIDs must be specified here.\n"
-                "The specified GUIDs are assigned\n"
-                "to the following fields, repectively:\n"
+                "The specified GUIDs are assigned to the following fields, respectively:\n"
                 "node, port1, port2 and system image GUID.\n\n"
-                "Note: port2 guid must be specified even for a\n"
-                "single port HCA - The HCA ignores this value.\n"
+                "Note: port2 guid must be specified even for a single port HCA - The HCA ignores this value.\n"
                 "It can be set to 0x0.\n\n"
                 "Commands affected: burn, sg");
 
     AddOptions("mac",
                ' ',
                 "<MAC>",
-                "MAC address base value. 2 MACs\n"
-                "are automatically assigned to the\n"
-                "following values:\n\n"
+                "MAC address base value. 2 MACs are automatically assigned to the following values:\n\n"
                 "mac    -> port1\n"
                 "mac+1  -> port2\n\n"
                 "Commands affected: burn, sg");
@@ -492,8 +454,7 @@ void Flint::initCmdParser() {
                ' ',
                 "<MACs...>",
                 "2 MACs must be specified here.\n"
-                "The specified MACs are assigned\n"
-                "to port1, port2, repectively.\n"
+                "The specified MACs are assigned to port1, port2, respectively.\n"
                 "Commands affected: burn, sg\n\n"
                 "Note: -mac/-macs flags are applicable only for Mellanox\n"
                 "\tTechnologies ethernet products.");
@@ -501,32 +462,15 @@ void Flint::initCmdParser() {
     AddOptions("uid",
                ' ',
                 "<UID>",
-                "BridgeX/ConnectIB only. Derive and set the device UIDs (GUIDs, MACs, WWNs).\n"
+                "ConnectIB/SwitchIB only. Derive and set the device UIDs (GUIDs, MACs, WWNs).\n"
                 "UIDs are derived from the given base UID according to Mellanox Methodology\n"
                 "Commands affected: burn, sg");
 
-    AddOptions("uids",
-               ' ',
-                "<UIDs...>",
-                "BridgeX only. 29 space separated UIDs must be specified here.\n"
-                "The specified UIDs are assigned to the following fields, repectively:\n"
-                "G0-MAC-PI0      G0-MAC-PI1      G0-MAC-PI2\n"
-                "G0-MAC-PE0      G0-MAC-PE1      G0-MAC-PE2    G0-MAC-PE3\n"
-                "G0-FC-WWPN-P0   G0-FC-WWPN-P1   G0-FC-WWPN-P2 G0-FC-WWPN-P3\n"
-                "G0-IB-NODE-GUID G0-IB-PORT-GUID G0-FC-WWNN\n"
-                "G1-MAC-PI0      G1-MAC-PI1      G1-MAC-PI2\n"
-                "G1-MAC-PE0      G1-MAC-PE1      G1-MAC-PE2    G1-MAC-PE3\n"
-                "G1-FC-WWPN-P0   G1-FC-WWPN-P1   G1-FC-WWPN-P2 G1-FC-WWPN-P3\n"
-                "G1-IB-NODE-GUID G1-IB-PORT-GUID G1-FC-WWNN\n"
-                "IB-SYSTEM-GUID\n"
-                "Commands affected: burn, sg");
-
     AddOptions("blank_guids",
                ' ',
                 "",
-                "Burn the image with blank GUIDs and MACs (where\n"
-                "applicable). These values can be set later using\n"
-                "the \"sg\" command (see details below).\n\n"
+                "Burn the image with blank GUIDs and MACs (where applicable). These values can be set later using the"
+                " \"sg\" command (see details below).\n\n"
                 "Commands affected: burn");
 
     AddOptions("clear_semaphore",
@@ -534,17 +478,15 @@ void Flint::initCmdParser() {
                 "",
                 "Force clear the flash semaphore on the device.\n"
                 "No command is allowed when this flag is used.\n"
-                "NOTE: May result in system instability or flash\n"
-                "\tcorruption if the device or another\n"
-                "\tapplication is currently using the flash.\n"
-                "\tExercise caution.\n");
+                "NOTE: May result in system instability or flash corruption if the device or another application is"
+                " currently using the flash.\n"
+                "Exercise caution.\n");
 
     AddOptions("qq",
                ' ',
                 "",
-                "Run a quick query. When specified, flint will not perform full\n"
-                "image integrity checks during the query operation. This may shorten\n"
-                "execution time when running over slow interfaces (e.g., I2C, MTUSB-1).\n"
+                "Run a quick query. When specified, flint will not perform full image integrity checks during the query"
+                " operation. This may shorten execution time when running over slow interfaces (e.g., I2C, MTUSB-1).\n"
                 "Commands affected: burn, query");
 
     AddOptions("nofs",
@@ -555,9 +497,8 @@ void Flint::initCmdParser() {
     AddOptions("allow_psid_change",
                ' ',
                 "",
-                "Allow burning a FW image with a different PSID (Parameter Set ID)than the\n"
-                "one currently on flash. Note that changing a PSID may cause the device to\n"
-                "malfunction. Use only if you know what you are doing");
+                "Allow burning a FW image with a different PSID (Parameter Set ID)than the one currently on flash. Note"
+                " that changing a PSID may cause the device to malfunction. Use only if you know what you are doing", isExternal);
 
     AddOptions("allow_rom_change",
                ' ',
@@ -571,13 +512,18 @@ void Flint::initCmdParser() {
                 "On SwitchX/ConnectIB devices:\n"
                 "Allow accessing the flash even if the cache replacement mode is enabled.\n"
                 "NOTE: This flag is intended for advanced users only.\n"
-                "\tRunning in this mode may cause the firmware to hang.\n");
+                "Running in this mode may cause the firmware to hang.\n");
 
     AddOptions("no_flash_verify",
                ' ',
                 "",
                 "Do not verify each write on the flash.");
 
+    AddOptions("use_fw",
+               ' ',
+                "",
+                "Flash access will be done using FW (ConnectX-3/ConnectX-3Pro only).");
+
     AddOptions("silent",
                's',
                 "",
@@ -598,7 +544,7 @@ void Flint::initCmdParser() {
     AddOptions("use_image_guids",
                ' ',
                 "",
-                "Burn (guids/uids/macs) as appears in the given image.\n"
+                "Burn (guids/macs) as appears in the given image.\n"
                 "Commands affected: burn");
 
     AddOptions("use_image_rom",
@@ -607,11 +553,36 @@ void Flint::initCmdParser() {
                 "Do not save the ROM which exists in the device.\n"
                 "Commands affected: burn");
 
+    AddOptions("use_dev_rom",
+               ' ',
+                "",
+                "Save the ROM which exists in the device.\n"
+                "Commands affected: burn");
+
+    AddOptions("ignore_dev_data",
+               ' ',
+                "",
+                "Do not attempt to take device data sections from device(sections will be taken from the image. FS3"
+                " Only).\n"
+                "Commands affected: burn");
+
+    AddOptions("no_fw_ctrl",
+               ' ',
+                "",
+                "Do not attempt to work with the FW Ctrl update commands");
+
+    AddOptions("use_dev_img_info",
+               ' ',
+                "",
+                "preserve select image info fields from the device upon FW update (FS3 Only).\n"
+                "Commands affected: burn",
+                true);
+
     AddOptions("dual_image",
                ' ',
                 "",
-                "Make the burn process burn two images on flash (previously default algorithm). Current"
-                "default failsafe burn process burns a single image (in alternating locations).\n"
+                "Make the burn process burn two images on flash (previously default algorithm). Current default"
+                " failsafe burn process burns a single image (in alternating locations).\n"
                 "Commands affected: burn");
 
     AddOptions("striped_image",
@@ -633,11 +604,13 @@ void Flint::initCmdParser() {
     char flashList[FLASH_LIST_SZ];
     char flashParDesc[FLASH_LIST_SZ*2];
     Flash::get_flash_list(flashList);
-    snprintf(flashParDesc, FLASH_LIST_SZ*2,"Use the given parameters to access the flash instead of reading them from the flash.\n"\
+    snprintf(flashParDesc, FLASH_LIST_SZ*2,"Use the given parameters to access the flash instead of reading them from "
+                                           "the flash.\n"\
             							   "Supported parameters:\n"\
             							   "Type: The type of the flash, such as:%s.\n"\
             							   "log2size: The log2 of the flash size."\
-            							   "num_of_flashes: the number of the flashes connected to the device.", flashList);
+            							   "num_of_flashes: the number of the flashes connected to the device.",
+                                            flashList);
 
 
     AddOptions("flash_params",
@@ -654,10 +627,64 @@ void Flint::initCmdParser() {
                 "",
                 "ignore device_id checks", true);
 
+    AddOptions("skip_ci_req",
+               ' ',
+                "",
+                "skip sending cache image request to driver(windows)", true);
+
     AddOptions("ocr",
                ' ',
                 "",
                 "another flag for override cache replacement", true);
+    AddOptions("private_key",
+               ' ',
+                "<key_file>",
+                "path to PEM formatted private key to be used by the sign command");
+
+    AddOptions("key_uuid",
+               ' ',
+                "<uuid_file>",
+                "UUID matching the given private key to be used by the sign command");
+
+    for (map_sub_cmd_t_to_subcommand::iterator it=_subcommands.begin(); it != _subcommands.end(); it++)
+        {
+            if (it->first == SC_ResetCfg) {
+                // hidden command so "forget" mentioning it
+                continue;
+            }
+            string str1 = it->second->getFlagL()+((it->second->getFlagS() == "") ? ("  ") :
+                          ("|"))+it->second->getFlagS()\
+                    +" "+it->second->getParam();
+            string str2 = it->second->getDesc();
+
+            AddOptionalSectionData("COMMANDS SUMMARY", str1, str2);
+        }
+    
+    AddOptionalSectionData("RETURN VALUES", "0", "Successful completion.");
+    AddOptionalSectionData("RETURN VALUES", "1", "An error has occurred.");
+    AddOptionalSectionData("RETURN VALUES", "7", "For burn command - FW already updated - burn was aborted.");
+
+
+    for (map_sub_cmd_t_to_subcommand::iterator it=_subcommands.begin(); it != _subcommands.end(); it++)
+        {
+            if (it->first == SC_ResetCfg) {
+                // hidden command so "forget" mentioning it
+                continue;
+            }
+            string str =   "Name:\n"
+                         "\t" + it->second->getName() + "\n"
+                         + "Description:\n"
+                         "\t" + it->second->getExtDesc() + "\n"
+                         + "Command:\n"
+                         "\t" + it->second->getFlagL() +
+                         ((it->second->getFlagS() == "") ? ("  ") : ("|"))+it->second->getFlagS()
+                         + " " + it->second->getParam() + "\n"
+                         + "Parameters:\n" 
+                         "\t" + it->second->getParamExp() + "\n"
+                         + "Examples:\n"
+                         "\t" + it->second->getExample() + "\n\n\n";
+            AddOptionalSectionData("COMMANDS DESCRIPTION", str);
+        }
 
 
     _cmdParser.AddRequester(this);
@@ -677,21 +704,9 @@ ParseStatus Flint::HandleOption(string name, string value)
     }
     else if (name == "help" || name == "h")
     {
-        cout<< _cmdParser.GetUsage()<<endl;
-        cout<<"COMMANDS SUMMARY:"<<endl;
-        for (map_sub_cmd_t_to_subcommand::iterator it=_subcommands.begin(); it != _subcommands.end(); it++)
-        {
-			if (it->first == SC_ResetCfg) {
-				// hidden command so "forget" mentioning it
-				continue;
-			}
-            string str1 = "  "+it->second->getFlagL()+((it->second->getFlagS() == "") ? ("  ") : ("|"))+it->second->getFlagS()\
-                    +" "+it->second->getParam();
-            string str2 = ": " + it->second->getDesc()+"\n";
-            printf("%-40s %s", str1.c_str(), str2.c_str());
-        }
-        cout<<endl<<"  Return values:\n  0 - Successful completion\n  1 - An error has occurred\n"
-                "  7 - For burn command - FW already updated - burn was aborted."<<endl << endl;
+        vector<string> excluded_sections;
+        excluded_sections.push_back("COMMANDS DESCRIPTION");
+        cout<< _cmdParser.GetUsage(false, excluded_sections);
         return PARSE_OK_WITH_EXIT;
     }
     else if (name == "version" || name == "v")
@@ -705,36 +720,14 @@ ParseStatus Flint::HandleOption(string name, string value)
     }
     else if (name == "hh")
     {
-        cout << _cmdParser.GetUsage()<<endl;
-        cout<<"COMMANDS SUMMARY:"<<endl;
-        for (map_sub_cmd_t_to_subcommand::iterator it=_subcommands.begin(); it != _subcommands.end(); it++)
-            {
-                string str1 = "  "+it->second->getFlagL()+((it->second->getFlagS() == "") ? ("  ") : ("|"))+it->second->getFlagS()\
-                        +" "+it->second->getParam();
-                string str2 = ": " + it->second->getDesc()+"\n";
-                printf("%-40s %s", str1.c_str(), str2.c_str());
-            }
-        cout<<endl<<endl<<"COMMANDS DESCRIPTION:"<<endl;
-        for (map_sub_cmd_t_to_subcommand::iterator it=_subcommands.begin(); it != _subcommands.end(); it++)
-                {
-        			if (it->first == SC_ResetCfg) {
-        				// hidden command so "forget" mentioning it
-        				continue;
-        			}
-                    cout<<"  Name:"<<endl<<"\t"<<it->second->getName()<<endl\
-                            <<"  Description:"<<endl<<"\t"<<it->second->getExtDesc()<<endl\
-                            <<"  Command:"<<endl<<"\t"<<it->second->getFlagL()<<\
-                            ((it->second->getFlagS() == "") ? ("  ") : ("|"))+it->second->getFlagS()\
-                            <<" "<<it->second->getParam()<<endl\
-                            <<"  Parameters:"<<endl<<"\t"<<it->second->getParamExp()<<endl\
-                            <<"  Examples:"<<endl<<"\t"<<it->second->getExample()<<endl<<endl<<endl;
-                }
-                cout<<endl<<"  Return values:\n  0 - Successful completion\n  1 - An error has occurred\n"
-                        "  7 - For burn command - FW already updated - burn was aborted."<<endl<< endl;
+        cout << _cmdParser.GetUsage();
         return PARSE_OK_WITH_EXIT;
     }
     else if (name == "no_devid_check") {
-            _flintParams.no_devid_check = false;
+            _flintParams.no_devid_check = true;
+    }
+    else if (name == "skip_ci_req") {
+            _flintParams.skip_ci_req = true;
     }
     else if (name == "guid") {
         _flintParams.guid_specified = true;
@@ -792,25 +785,6 @@ ParseStatus Flint::HandleOption(string name, string value)
         if (!getGUIDFromStr(value, _flintParams.baseUid)) {
             return PARSE_ERROR;
         }
-        if (!InitBxGuids( _flintParams.user_uids, _flintParams.baseUid)) {
-            printf("-E- failed to extract UIDs from UID.");
-            return PARSE_ERROR;
-        }
-    } else if (name == "uids") {
-        _flintParams.uids_specified = true;
-        std::vector<std::string> strs;
-        splitByDelimiters(strs, value,",");
-        if (strs.size() < BX_ALL_GUIDS) {
-        	return PARSE_ERROR;
-        }
-        for (int i = 0; i < BX_ALL_GUIDS; i++) {
-            guid_t u;
-            if (!getGUIDFromStr(strs[i], u)){
-                return PARSE_ERROR;
-            }else{
-                _flintParams.user_uids.push_back(u);
-            }
-        }
     } else if (name == "blank_guids") {
         _flintParams.blank_guids = true;
     } else if (name == "clear_semaphore") {
@@ -828,6 +802,8 @@ ParseStatus Flint::HandleOption(string name, string value)
         _flintParams.allow_rom_change = true;
     } else if (name == "override_cache_replacement" || name == "ocr") {
         _flintParams.override_cache_replacement = true;
+    } else if (name == "use_fw") {
+        _flintParams.use_fw = true;
     } else if (name == "no_flash_verify") {
         _flintParams.no_flash_verify = true;
     } else if (name == "silent" || name == "s") {
@@ -845,11 +821,19 @@ ParseStatus Flint::HandleOption(string name, string value)
         _flintParams.use_image_guids = true;
     } else if (name == "use_image_rom") {
         _flintParams.use_image_rom = true;
+    } else if (name == "use_dev_rom") {
+        _flintParams.use_dev_rom = true;
+    } else if (name == "ignore_dev_data") {
+        _flintParams.ignore_dev_data = true;
+    } else if (name == "no_fw_ctrl") {
+        _flintParams.no_fw_ctrl = true;
     } else if (name == "dual_image") {
         _flintParams.dual_image = true;
     } else if (name == "striped_image") {
         _flintParams.striped_image = true;
-    } else if (name == "banks") {
+    }else if (name == "use_dev_img_info") {
+        _flintParams.use_dev_img_info = true;
+    }else if (name == "banks") {
         _flintParams.banks_specified = true;
         u_int64_t banksNum;
         if (!strToNum(value, banksNum)) {
@@ -865,13 +849,22 @@ ParseStatus Flint::HandleOption(string name, string value)
             return PARSE_ERROR;
         }
         //printf("-D- flashType=%s , log2size = %d , numOfBanks = %d\n", _flintParams.flash_params.type_name, _flintParams.flash_params.log2size, _flintParams.flash_params.num_of_flashes);
+    } else if (name == "private_key") {
+        _flintParams.privkey_specified = true;
+        _flintParams.privkey_file = value;
+    } else if (name == "key_uuid") {
+        _flintParams.uuid_specified = true;
+        _flintParams.privkey_uuid = value;
     } else {
         cout << "Unknown Flag: " << name;
+        cout<< _cmdParser.GetSynopsis();
         return PARSE_ERROR;
     }
     return PARSE_OK;
 }
 
+#define IS_NUM(cha) (((cha) >= '0') && ((cha) <= '9'))
+
 ParseStatus Flint::parseCmdLine(int argc, char* argv[]) {
     //Step1 separate between option section and cmd section
     SubCmdMetaData subCmds;
@@ -895,14 +888,14 @@ ParseStatus Flint::parseCmdLine(int argc, char* argv[]) {
     //printf("-D- argcOpt:%d argvOpt:%s argcCmd:%d argvCmd:%s\n", argcOpt, argvOpt[0], argcCmd, argvCmd[0]);
     //_cmdparser should deal with the case of no arguments in argv except the program.
     //Step2 unite with comma multiple args in the options section
-    char* newArgv[argcOpt];
+    char** newArgv = new char*[argcOpt];
     int newArgc = 0;
     int i = 1, j = 1, argStart = 1, argEnd = 1;
     //first arg is the flint command we can copy as is
     newArgv[0] = strcpy(new char[strlen(argvOpt[0]) + 1], argvOpt[0]);
     while (i < argcOpt) {
-        if (argvOpt[i][0] == '-') //its a flag so we copy to new_argv as is
-                {
+        if (argvOpt[i][0] == '-' && !IS_NUM(argvOpt[i][1])) //its a flag (and not a number) so we copy to new_argv as is
+        {
             newArgv[j] = strcpy(new char[strlen(argvOpt[i]) + 1],
                     argvOpt[i]);
             i++;
@@ -913,7 +906,7 @@ ParseStatus Flint::parseCmdLine(int argc, char* argv[]) {
             argStart = i;
             argEnd = i;
             int argsSize = 0;
-            while (argEnd < argcOpt && argvOpt[argEnd][0] != '-') {
+            while (argEnd < argcOpt && (argvOpt[argEnd][0] != '-' || IS_NUM(argvOpt[argEnd][1]))) {
                 argsSize += strlen(argvOpt[argEnd]) + 1; //for the comma
                 argEnd++;
             }
@@ -949,7 +942,7 @@ ParseStatus Flint::parseCmdLine(int argc, char* argv[]) {
         int lastFlagPos;
         char* strippedFlag;
         for (lastFlagPos = argc - 1; lastFlagPos > 0; lastFlagPos--) {
-            if (argv[lastFlagPos][0] == '-')
+            if (argv[lastFlagPos][0] == '-' && !IS_NUM(argv[lastFlagPos][1]))
                 break;
         }
         if (lastFlagPos == 0) {
@@ -964,6 +957,7 @@ ParseStatus Flint::parseCmdLine(int argc, char* argv[]) {
         if ((argc - 1 - lastFlagPos) > numOfArgs) {
             printf(FLINT_INVALID_COMMAD_ERROR, argv[argc - 1]);
             rc = PARSE_ERROR;
+            cout<< _cmdParser.GetSynopsis();
             goto clean_up;
         }
     }
@@ -973,5 +967,6 @@ ParseStatus Flint::parseCmdLine(int argc, char* argv[]) {
     clean_up: for (int i = 0; i < newArgc; i++) {
         delete[] newArgv[i];
     }
+    delete[] newArgv;
     return rc;
 }
