@@ -89,7 +89,7 @@ protected:
     string _example;
     char  _errBuff[FLINT_ERR_LEN];
     sub_cmd_t _cmdType;
-
+    bool _mccSupported;
 
     //Methods that are commonly used in the various subcommands:
     //TODO: add middle classes and segregate as much of these common methods between these classes
@@ -104,11 +104,14 @@ protected:
     bool getRomsInfo(FBase* io, roms_info_t& romsInfo);
     void displayOneExpRomInfo(const rom_info_t& info);
     void displayExpRomInfo(const roms_info_t& romsInfo, const char *preStr);
+    string getRomProtocolStr(u_int8_t proto);
+    string getRomSuppCpuStr(u_int8_t suppCpu);
 
     static int verifyCbFunc(char* str);
     static int CbCommon(int completion, char*preStr, char* endStr=NULL);
     static int burnCbFs2Func(int completion);
     static int burnCbFs3Func(int completion);
+    static int advProgressFunc(int completion, const char* stage, prog_t type, int* unknownProgress);
     static int burnBCbFunc(int completion);
     static int vsdCbFunc(int completion);
     static int setKeyCbFunc(int completion);
@@ -149,7 +152,7 @@ protected:
 
 
 public:
-    SubCommand(): _fwOps(NULL), _imgOps(NULL), _io(NULL), _v(Wtv_Uninitilized), _maxCmdParamNum(-1),  _minCmdParamNum(-1)
+    SubCommand(): _fwOps(NULL), _imgOps(NULL), _io(NULL), _v(Wtv_Uninitilized), _maxCmdParamNum(-1),  _minCmdParamNum(-1), _mccSupported(false)
     {
         _cmdType = SC_No_Cmd;
         memset(_errBuff, 0, sizeof(_errBuff));
@@ -175,6 +178,7 @@ private:
     fw_info_t _imgInfo;
     FwOperations::ExtBurnParams _burnParams;
     bool _devQueryRes;
+    int _unknownProgress; // used to trace the progress of unknown progress.
 
     FlintStatus burnFs3();
     FlintStatus burnFs2();
@@ -195,6 +199,7 @@ public:
 class QuerySubCommand : public SubCommand
 {
 private:
+    string printSecurityAttrInfo(u_int32_t m);
     FlintStatus printInfo(const fw_info_t& fwInfo, bool fullQuery);
     bool displayFs3Uids(const fw_info_t& fwInfo);
     bool displayFs2Uids(const fw_info_t& fwInfo);
@@ -206,6 +211,44 @@ public:
     bool verifyParams();
 };
 
+class Extract4MBImageSubCommand : public SubCommand
+{
+private:
+public:
+    Extract4MBImageSubCommand();
+    ~Extract4MBImageSubCommand();
+    FlintStatus executeCommand();
+};
+
+
+class SignSubCommand : public SubCommand
+{
+private:
+public:
+    SignSubCommand();
+    ~SignSubCommand();
+    FlintStatus executeCommand();
+    bool verifyParams();
+};
+
+class SetPublicKeysSubCommand : public SubCommand
+{
+private:
+public:
+    SetPublicKeysSubCommand();
+    ~SetPublicKeysSubCommand();
+    FlintStatus executeCommand();
+};
+
+class SetForbiddenVersionsSubCommand : public SubCommand
+{
+private:
+public:
+    SetForbiddenVersionsSubCommand();
+    ~SetForbiddenVersionsSubCommand();
+    FlintStatus executeCommand();
+};
+
 class VerifySubCommand : public SubCommand
 {
 private:
