@@ -1,6 +1,4 @@
 /*
- *  mtcr.h - Mellanox Software tools (mst) driver definitions
- *
  * Copyright (C) Jan 2013 Mellanox Technologies Ltd. All rights reserved.
  *
  * This software is available to you under a choice of one of two
@@ -12,7 +10,7 @@
  *     Redistribution and use in source and binary forms, with or
  *     without modification, are permitted provided that the following
  *     conditions are met:
- * 
+ *
  *      - Redistributions of source code must retain the above
  *        copyright notice, this list of conditions and the following
  *        disclaimer.
@@ -21,7 +19,7 @@
  *        copyright notice, this list of conditions and the following
  *        disclaimer in the documentation and/or other materials
  *        provided with the distribution.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
@@ -42,144 +40,15 @@ extern "C" {
 
 #include <sys/types.h>
 
-typedef enum MError {
-    ME_OK = 0,
-    ME_ERROR,
-    ME_BAD_PARAMS,
-    ME_CR_ERROR,
-    ME_NOT_IMPLEMENTED,
-    ME_SEM_LOCKED,
-    ME_MEM_ERROR,
+#include "mtcr_com_defs.h"
+#include "mtcr_mf.h"
 
-    ME_MAD_SEND_FAILED,
-    ME_UNKOWN_ACCESS_TYPE,
-    ME_UNSUPPORTED_DEVICE,
-    ME_REG_NOT_SUPPORTED,
-
-    // errors regarding REG_ACCESS
-    ME_REG_ACCESS_OK = 0,
-    ME_REG_ACCESS_BAD_STATUS_ERR = 0x100,
-    ME_REG_ACCESS_BAD_METHOD,
-    ME_REG_ACCESS_NOT_SUPPORTED,
-    ME_REG_ACCESS_DEV_BUSY,
-    ME_REG_ACCESS_VER_NOT_SUPP,
-    ME_REG_ACCESS_UNKNOWN_TLV,
-    ME_REG_ACCESS_REG_NOT_SUPP,
-    ME_REG_ACCESS_CLASS_NOT_SUPP,
-    ME_REG_ACCESS_METHOD_NOT_SUPP,
-    ME_REG_ACCESS_BAD_PARAM,
-    ME_REG_ACCESS_RES_NOT_AVLBL,
-    ME_REG_ACCESS_MSG_RECPT_ACK,
-    ME_REG_ACCESS_UNKNOWN_ERR,
-    ME_REG_ACCESS_SIZE_EXCCEEDS_LIMIT,
-    ME_REG_ACCESS_CONF_CORRUPT,
-    ME_REG_ACCESS_LEN_TOO_SMALL,
-    ME_REG_ACCESS_BAD_CONFIG,
-    ME_REG_ACCESS_ERASE_EXEEDED,
-
-    // errors regarding ICMD
-    ME_ICMD_STATUS_CR_FAIL = 0x200,       // cr-space access failure
-    ME_ICMD_INVALID_OPCODE,
-    ME_ICMD_INVALID_CMD,
-    ME_ICMD_OPERATIONAL_ERROR,
-    ME_ICMD_BAD_PARAM,
-    ME_ICMD_BUSY,
-    ME_ICMD_INIT_FAILED,
-    ME_ICMD_NOT_SUPPORTED,
-    ME_ICMD_STATUS_SEMAPHORE_TO,          // timed out while trying to take semaphore
-    ME_ICMD_STATUS_EXECUTE_TO,            // timed out while waiting for command to execute
-    ME_ICMD_STATUS_IFC_BUSY,
-    ME_ICMD_STATUS_ICMD_NOT_READY,
-    ME_ICMD_UNSUPPORTED_ICMD_VERSION,
-    ME_ICMD_UNKNOWN_STATUS,
-    ME_ICMD_ICM_NOT_AVAIL,
-    ME_ICMD_WRITE_PROTECT,
-
-    //errors regarding Tools CMDIF
-    ME_CMDIF_BUSY = 0x300,
-    ME_CMDIF_TOUT,
-    ME_CMDIF_BAD_STATUS,
-    ME_CMDIF_BAD_OP,
-    ME_CMDIF_NOT_SUPP,
-    ME_CMDIF_BAD_SYS,
-
-    ME_LAST
-} MError;
-
-typedef enum Mdevs_t {
-    MDEVS_GAMLA     = 0x01, /*  Each device that actually is a Gamla */
-    MDEVS_I2CM      = 0x02, /*  Each device that can work as I2C master */
-    MDEVS_MEM       = 0x04, /*  Each device that is a memory driver (vtop) */
-    MDEVS_TAVOR_DDR = 0x08, /*  Each device that maps to Tavor DDR */
-    MDEVS_TAVOR_UAR = 0x10, /*  Each device that maps to Tavor UAR */
-    MDEVS_TAVOR_CR  = 0x20, /*  Each device that maps to Tavor CR */
-    MDEVS_IF        = 0x40, /*  Standard device  interface */
-    MDEVS_REM       = 0x80, /*  Remote devices */
-    MDEVS_PPC       = 0x100, /*  PPC devices */
-    MDEVS_DEV_I2C   = 0x200, /* Generic linux kernel i2c device */
-    MDEVS_IB        = 0x400, /* Cr access over IB Mads */
-    MDEVS_MLNX_OS   = 0x800, /* access by CmdIf in MlnxOS */
-    MDEVS_FWCTX		= 0x900, /*access by UEFI func/context */
-    MDEVS_TAVOR     = (MDEVS_TAVOR_DDR|MDEVS_TAVOR_UAR|MDEVS_TAVOR_CR),
-    MDEVS_ALL       = 0xffffffff
-} Mdevs;
-typedef struct mfile_t mfile;
-
-typedef enum {
-    MACCESS_REG_METHOD_GET = 1,
-    MACCESS_REG_METHOD_SET = 2
-} maccess_reg_method_t;
-
-typedef struct dev_info_t
-{
-    Mdevs         type;
-    char          dev_name[512];
-
-    union {
-        struct {
-            u_int16_t domain;
-            u_int8_t  bus;
-            u_int8_t  dev;
-            u_int8_t  func;
-
-            u_int16_t dev_id;
-            u_int16_t vend_id;
-            u_int32_t class_id;
-            u_int16_t subsys_id;
-            u_int16_t subsys_vend_id;
-
-            char      cr_dev[512];
-            char      conf_dev[512];
-            char**    net_devs;      // Null terminated array
-            char**    ib_devs;       // Null terminated array
-        } pci;
-
-        struct {
-            u_int32_t TBD;
-        } usb;
-
-        struct {
-            u_int32_t TBD;
-        } ib;
-
-        struct {
-            u_int32_t TBD;
-        } remote;
-    };
-} dev_info;
-
-typedef enum {
-    RA_MFPA=0x9010,
-    RA_MFBA=0x9011,
-    RA_MFBE=0x9012,
-} reg_access_t;
-
-enum mtcr_access_method {
-    MTCR_ACCESS_ERROR  = 0x0,
-    MTCR_ACCESS_MEMORY = 0x1,
-    MTCR_ACCESS_CONFIG = 0x2,
-    MTCR_ACCESS_INBAND = 0x3
-};
+typedef enum mtcr_access_method {
+    MTCR_ACCESS_ERROR  = MST_ERROR,
+    MTCR_ACCESS_MEMORY = MST_PCI,
+    MTCR_ACCESS_CONFIG = MST_PCICONF,
+    MTCR_ACCESS_INBAND = MST_IB
+} mtcr_access_method_t;
 /*
  * Read 4 bytes, return number of succ. read bytes or -1 on failure
  */
@@ -195,6 +64,7 @@ int mread4_block (mfile *mf, unsigned int offset, u_int32_t* data, int byte_len)
 int mwrite4_block (mfile *mf, unsigned int offset, u_int32_t* data, int byte_len);
 
 int msw_reset(mfile *mf);
+int mhca_reset(mfile *mf);
 
 /*
  * Get list of MST (Mellanox Software Tools) devices.
@@ -213,20 +83,31 @@ int mdevices(char *buf, int len, int mask);
  */
 dev_info* mdevices_info(int mask, int* len);
 
+/*
+ *  * Get list of MST (Mellanox Software Tools) devices info records.
+ *  * Return a dynamic allocated array of dev_info records.
+ *  * len will be updated to hold the array length
+ *  * Verbosity will decide whether to get all the Physical functions or not.
+ */
+
+dev_info* mdevices_info_v(int mask, int* len, int verbosity);
+
 void mdevice_info_destroy(dev_info* dev_info, int len);
 void mdevices_info_destroy(dev_info* dev_info, int len);
 
 int mget_mdevs_type(mfile *mf, u_int32_t *mtype);
 
 /*
- * Open Mellanox Software tools (mst) driver. Device type==TAVOR
+ * Open Mellanox Software tools (mst) driver. Device type==INFINIHOST
  * Return valid mfile ptr or 0 on failure
  */
 mfile *mopen(const char *name);
 
 mfile *mopend(const char *name, int type);
 
-mfile *mopen_fw_ctx(void* fw_cmd_context, void* fw_cmd_func);
+mfile *mopen_fw_ctx(void* fw_cmd_context, void* fw_cmd_func, void* extra_data);
+
+mfile *mopen_adv(const char *name, MType mtype);
 
 /*
  * Close Mellanox driver
@@ -258,6 +139,16 @@ int icmd_send_command(mfile *mf, int opcode, void* data, int data_size, int skip
 
 int icmd_clear_semaphore(mfile *mf);
 
+
+int tools_cmdif_send_inline_cmd(mfile* mf, u_int64_t in_param, u_int64_t* out_param,
+                                u_int32_t input_modifier, u_int16_t opcode, u_int8_t  opcode_modifier);
+
+int tools_cmdif_send_mbox_command(mfile* mf, u_int32_t input_modifier, u_int16_t opcode, u_int8_t  opcode_modifier,
+                                  int data_offs_in_mbox, void* data, int data_size, int skip_write);
+
+int tools_cmdif_unlock_semaphore(mfile *mf);
+
+
 int mget_max_reg_size(mfile *mf);
 
 const char* m_err2str(MError status);
@@ -265,7 +156,19 @@ const char* m_err2str(MError status);
 int mread_buffer(mfile *mf, unsigned int offset, u_int8_t* data, int byte_len);
 int mwrite_buffer(mfile *mf, unsigned int offset, u_int8_t* data, int byte_len);
 
-int tools_cmdif_query_dev_cap(mfile *mf, u_int32_t offset, u_int64_t* data);
+int mget_vsec_supp(mfile* mf);
+
+int mget_addr_space(mfile* mf);
+int mset_addr_space(mfile* mf, int space);
+
+int mclear_pci_semaphore(const char* name);
+
+int mvpd_read4(mfile *mf, unsigned int offset, u_int8_t value[4]);
+
+#ifndef NO_INBAND
+int mib_smp_set(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod);
+int mib_smp_get(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod);
+#endif
 
 #ifdef __cplusplus
 }
