@@ -1,5 +1,9 @@
 /*
- * Copyright (C) Jan 2013 Mellanox Technologies Ltd. All rights reserved.
+ *
+ * mflash.h - Mellanox Technilogies LTD. Flash access lib heared file
+ * ==================================================================
+ *
+ * Copyright (c) 2005 Mellanox Technologies Ltd.  All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -28,16 +32,21 @@
  * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
+ *
+ *  Version: $Id: $
+ *
  */
 
-
 #ifndef MFLASH_H
 #define MFLASH_H
 
 #include <compatibility.h>
+#include <mtcr.h>
 #include "mflash_types.h"
 #include "mflash_common_structs.h"
 
+#include <mlxfwops/uefi_c/mft_uefi_common.h>
+
 #ifdef __cplusplus
 #define EXTERN_C_START extern "C" {
 #define EXTERN_C_END   }
@@ -56,22 +65,6 @@ typedef enum MfCommandSet {
     MCS_SSTSPI = 0x81
 } MfCommandSet;
 
-typedef struct flash_params {
-    const char *type_name;
-    int log2size;
-    int num_of_flashes;
-} flash_params_t;
-
-typedef struct write_protect_info {
-    u_int8_t is_subsector;
-    u_int8_t is_bottom;
-    u_int8_t sectors_num;
-} write_protect_info_t;
-
-// TODO: remove UEFI REMNANTS
-typedef struct _MLX4_DEV uefi_Dev_t;
-typedef int (*f_fw_cmd) (uefi_Dev_t* dev, void* buffer, int* size);
-
 /////////////////////////////////////////////
 //
 // MFLASH INTERFACE FUNCTIONS
@@ -108,11 +101,13 @@ int     mf_open        (mflash** pmfl, const char* dev, int num_of_banks, flash_
         int ignore_cache_rep_guard);
 int     mf_opend       (mflash** pmfl, struct mfile_t* mf, int num_of_banks,  flash_params_t* flash_params,
         int ignore_cache_rep_guard);
+int     mf_open_adv       (mflash** pmfl, const char* dev, int num_of_banks, flash_params_t* flash_params,
+        int ignore_cache_rep_guard, int cx3_fw_access);
 
-int     mf_open_uefi(mflash** pmfl, uefi_Dev_t *uefi_dev, f_fw_cmd fw_cmd_func);
+int     mf_open_uefi(mflash** pmfl, uefi_Dev_t *uefi_dev, uefi_dev_extra_t* dev_extra);
 
 int     mf_open_ignore_lock(mflash* mfl);
-int     mf_close       (mflash* mfl);
+void     mf_close       (mflash* mfl);
 
 //
 // Flash access functions:
@@ -120,7 +115,14 @@ int     mf_close       (mflash* mfl);
 //
 int     mf_read        (mflash* mfl, u_int32_t addr, u_int32_t len, u_int8_t* data);
 int     mf_write       (mflash* mfl, u_int32_t addr, u_int32_t len, u_int8_t* data);
-int     mf_erase_sector(mflash* mfl, u_int32_t addr);
+// mf_erase performs an erase on the smallest block
+int     mf_erase(mflash* mfl, u_int32_t addr);
+// adv erasing
+int     mf_erase_64k_sector(mflash* mfl, u_int32_t addr);
+int     mf_erase_4k_sector(mflash* mfl, u_int32_t addr);
+
+// swap between erasing 4KB or 64KB chunks (performance optimization)
+
 
 //
 // Crspace access through mflash:
@@ -153,7 +155,10 @@ int     mf_get_opt     (mflash* mfl, MfOpt opt, int *val);
 
 int     mf_enable_hw_access(mflash* mfl, u_int64_t key);
 int     mf_disable_hw_access(mflash* mfl);
-int     mf_release_semaphore();
+int     mf_release_semaphore(mflash* mfl);
+
+// get mfile object
+mfile* mf_get_mfile(mflash* mfl);
 
 //
 // err code to string translation for printing.
