@@ -1,6 +1,4 @@
 /*
- * mflash_common_structs.h - Mellanox Technilogies LTD. Flash access lib heared file
- * 
  * Copyright (C) Jan 2013 Mellanox Technologies Ltd. All rights reserved.
  *
  * This software is available to you under a choice of one of two
@@ -21,7 +19,7 @@
  *        copyright notice, this list of conditions and the following
  *        disclaimer in the documentation and/or other materials
  *        provided with the distribution.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
@@ -32,7 +30,6 @@
  * SOFTWARE.
  */
 
-
 #ifndef MFLASH_COMMON_STRUCTS_H
 #define MFLASH_COMMON_STRUCTS_H
 
@@ -50,6 +47,7 @@ typedef enum MfOpt {
     MFO_FW_ACCESS_TYPE_BY_MFILE,
     MFO_SX_TYPE,
     MFO_NEW_CACHE_REPLACEMENT_EN,
+    MFO_CX3_FW_ACCESS_EN,
     MFO_LAST
 } MfOpt;
 
@@ -58,6 +56,25 @@ enum MfAccessType {
     MFAT_UEFI,
 };
 
+typedef enum {
+    UNKNOWN_BIN = 0x0,
+    CX5_LOW_BIN  = 0x1,
+    CX5_HIGH_BIN = 0x2,
+
+} BinIdT;
+
+typedef struct flash_params {
+    const char *type_name;
+    int log2size;
+    int num_of_flashes;
+} flash_params_t;
+
+typedef struct write_protect_info {
+    u_int8_t is_subsector;
+    u_int8_t is_bottom;
+    u_int8_t sectors_num;  // if zero: is_subsector and is_bottom are invalid.
+} write_protect_info_t;
+
 /////////////////////////////////////////////
 //
 // Flash attributes struct
@@ -70,6 +87,7 @@ typedef struct flash_attr {
     //
     u_int32_t hw_dev_id;
     u_int32_t rev_id;
+    BinIdT    bin_id;
 
     //
     // size:        Total size (in bytes) of all flash devices connected to
@@ -84,11 +102,6 @@ typedef struct flash_attr {
     u_int32_t sector_size;
 
     int       num_erase_blocks;         // Number of sector defs.
-    struct {
-        unsigned long sector_size;      // Byte size of sector
-        int           num_sectors;      // Num sectors of this size
-        u_int32_t     sector_mask;      // Sector mask
-    } erase_block[8];
 
     //
     // bank_size:   Different bank means a different chip sellect or gpio settings is needed when crossing
@@ -125,9 +138,39 @@ typedef struct flash_attr {
     u_int8_t vendor;
     u_int8_t type;
 
+    u_int8_t support_sub_and_sector; // true if flash can work in both 64KB and 4KB sectors
+
 
 } flash_attr;
 
+typedef struct flash_info {
+    const char *name;
+    u_int8_t vendor;
+    u_int8_t type;
+    int command_set;
+    int erase_command;
+    u_int32_t sector_size;
+    u_int8_t support_sub_and_sector;
+    u_int8_t quad_en_support;
+    u_int8_t write_protected_support;
+    u_int8_t protect_sub_and_sector;
+    u_int8_t dummy_cycles_support;
+} flash_info_t;
+
+/*
+ * Common Macros:
+ */
+#define WRITE_PROTECT_CHECKS(mfl, bank_num) { \
+        if (!mfl->attr.write_protect_support) {\
+            return MFE_NOT_SUPPORTED_OPERATION;\
+        }\
+        if (bank_num >= mfl->attr.banks_num) {\
+            return MFE_FLASH_NOT_EXIST;\
+        }\
+}
+#define MAX_SUBSECTOR_NUM 8
+#define MAX_SECTORS_NUM   64
+
 #endif // MFLASH_COMMON_STRUCTS_H
 
 
