@@ -2,7 +2,7 @@
  *
  * flint_base.h - FLash INTerface
  *
- * Copyright (c) 2013 Mellanox Technologies Ltd.  All rights reserved.
+ * Copyright (c) 2011 Mellanox Technologies Ltd.  All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -31,6 +31,9 @@
  * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
+ *
+ *  Version: $Id: flint_base.h 7522 2011-11-16 15:37:21Z mohammad $
+ *
  */
 #ifndef FLINT_BASE_H
 #define FLINT_BASE_H
@@ -43,6 +46,10 @@
     #include "uefi_c.h"
 #endif
 
+#if defined __FreeBSD__
+    #include <stdarg.h>
+#endif
+
 #include <signal.h>
 #include "tools_version.h"
 
@@ -68,18 +75,6 @@
 // djgpp stdio does not define vsnprintf. I simply call vsprintf (and pray ...)
         #define vsnprintf(buf, len, format, args) (vsprintf(buf, format, args))
         #define snprintf(buf, len, format, args...) (sprintf(buf, format, args))
-    #else // Linux GCC
-        #ifdef __FreeBSD__
-            #define SWAPL(l) ntohl(l)
-            #include <sys/endian.h>
-            #define bswap_32(x) ntohl(x)
-        #else // Linux
-            #include <byteswap.h>
-            #include <endian.h>
-        #endif
-        #include <netinet/in.h>
-        #include <unistd.h>
-
     #endif // __DJGPP__
 
 #else // __WIN__
@@ -87,9 +82,6 @@
 //
 // Windows (Under DDK)
 //
-
-    #include <io.h>
-    #include <Winsock2.h>
     #include <mtcr.h>
 // Sleep adaptor
     #define usleep(x) Sleep((x)/1000)
@@ -98,25 +90,9 @@
     #define vsnprintf      _vsnprintf
     #define isatty         _isatty
 
-    #define COMP_CDECL     __cdecl
-
-    #define __LITTLE_ENDIAN 1234
-    #define __BIG_ENDIAN 4321
-    #define __BYTE_ORDER __LITTLE_ENDIAN
-
-
-
-    #if __BYTE_ORDER == __LITTLE_ENDIAN
-        #define bswap_32(x) ntohl(x)
-    #else
-        #error windows is assumed to run a on little endian architecture
-    #endif
-
     // MINGW
     #if defined(__MINGW32__) || (__MINGW64__)
-        #define strtoull       strtoull
         #define _UNISTD_H // Zlib includes unistd.h which causes some compilation errors.
-
     #else
         #define strtoull       _strtoui64
     #endif
@@ -124,7 +100,7 @@
 
 
 #include <vector>
-#include "compatibility.h"
+#include <compatibility.h>
 #include "mlxfwops_com.h"
 
 static inline void be_guid_to_cpu(guid_t* to, guid_t* from) {
@@ -210,29 +186,46 @@ namespace std {}; using namespace std;
 		return false; }} while (0)
 #define READBUF(f,o,d,l,p) do { if (!f.read(o,d,l)) { \
     return errmsg("%s - read error (%s)\n", p, f.err()); }} while (0)
+#define READALLOCBUF(f,o,d,l,p) do {\
+    d = (u_int8_t*) malloc(sizeof(u_int8_t) * l);\
+    if (!f.read(o,d,l)) { \
+        free(d);\
+        return errmsg("%s - read error (%s)\n", p, f.err());\
+    }\
+} while (0)
 
 #define FS2_BOOT_START   0x38
 #define FS_DATA_OFF      0x28
 #define SWITCHX_HW_ID    581
 #define SWITCH_IB_HW_ID  583
+#define SPECTRUM_HW_ID   585
+#define SWITCH_IB2_HW_ID 587
+#define QUANTUM_HW_ID    589
+#define SPECTRUM2_HW_ID  591
 
-#define CX4_HW_ID		  521
+
+#define CX4_HW_ID         521
+#define CX4LX_HW_ID       523
+#define CX5_HW_ID         525
+#define CX6_HW_ID         527
+#define BF_HW_ID          529
+#define CX2_HW_ID         400
 #define CX3_HW_ID         501
 #define CX3_PRO_HW_ID     503
-#define CX_HW_ID          400
-#define IS_HW_ID          435
-#define CONNECT_IB_HW_ID  511
-#define BRIDGEX_HW_ID     6100
 #define IS4_HW_ID         435
-#define TAVOR_HW_ID       23108
-#define ARBEL_HW_ID       25208
-#define SINAI_HW_ID       25204
+#define CONNECT_IB_HW_ID  511
 #define CONNECT_IB_SW_ID  4113
 
 // FS3 defines
 #define FS3_BOOT_START        FS2_BOOT_START
 #define FS3_BOOT_START_IN_DW  FS3_BOOT_START/4
 
+// FS4 defines
+#define FS4_BOOT_START        0x10
+#define FS4_BOOT_START_IN_DW  FS4_BOOT_START/4
+#define FS4_BOOT2_START        0x188
+#define FS4_HW_PTR_START       0x18
+#define FS4_HW_PTR_SIZE_DW     CX5FW_HW_POINTERS_SIZE / 4
 
 #define CRC_CHECK_OLD "%s /0x%08x-0x%08x (0x%06x)/ (%s"
 
@@ -304,24 +297,35 @@ typedef enum fs3_section {
     FS3_UPGRADE_CODE  = 0x7,
     FS3_HW_BOOT_CFG   = 0x8,
     FS3_HW_MAIN_CFG   = 0x9,
+    FS3_PHY_UC_CODE   = 0xa,
+    FS3_PHY_UC_CONSTS = 0xb,
+    FS3_PHY_UC_CMD    = 0xc,
     FS3_IMAGE_INFO    = 0x10,
     FS3_FW_BOOT_CFG   = 0x11,
     FS3_FW_MAIN_CFG   = 0x12,
     FS3_ROM_CODE      = 0x18,
     FS3_RESET_INFO    = 0x20,
-    FS3_DBG_LOG_MAP   = 0x30,
-    FS3_DBG_FW_INI    = 0x31,
+    FS3_DBG_FW_INI    = 0x30,
+    // FS3_DBG_LOG_MAP = 0x30 - in practice its unused and DBG_FW_INI is found in that section instead
     FS3_DBG_FW_PARAMS = 0x32,
     FS3_FW_ADB        = 0x33,
+    FS3_IMAGE_SIGNATURE = 0xa0,
+    FS3_PUBLIC_KEYS   = 0xa1,
+    FS3_FORBIDDEN_VERSIONS = 0xa2,
     FS3_MFG_INFO      = 0xe0,
     FS3_DEV_INFO      = 0xe1,
-    FS3_NV_DATA       = 0xe2,
+    FS3_NV_DATA1      = 0xe2,
     FS3_VPD_R0        = 0xe3,
+    FS3_NV_DATA2      = 0xe4,
+    FS3_FW_NV_LOG     = 0xe5,
+    FS3_NV_DATA0      = 0xe6, // replaces FS3_NV_DATA1
+    FS4_HW_PTR        = 0xfb,
+    FS4_TOOLS_AREA    = 0xfc,
     FS3_ITOC          = 0xfd,
+    FS3_DTOC          = 0xfe,
     FS3_END           = 0xff,
 } fs3_section_t;
 
-
 enum CommandType {
     CMD_UNKNOWN,
     CMD_BURN,
@@ -356,70 +360,12 @@ enum CommandType {
     CMD_HW_INFO,
     CMD_SET_MFG_GUIDS,
     CMD_BURN_VPD,
-};
-enum CommandInput {
-    CI_NONE         = 0x01,
-    CI_IMG_ONLY     = 0x02,
-    CI_DEV_ONLY     = 0x04,
-    CI_IMG_OR_DEV   = 0x06,
-    CI_IMG_AND_DEV  = 0x08
+    CMD_SET_SIGNATURE,
+    CMD_SET_PUBLIC_KEYS,
+    CMD_SET_FORBIDDEN_VERSIONS
 };
 
 
-
-struct CommandInfo {
-    CommandType  cmd;
-    const char*  cmdName;
-    bool         requireExactMatch;
-    int          maxArgs;
-    CommandInput requiredInput;
-    const char*  cmdDescription;
-
-};
-
-
-////////////////////////////////////////////////////////////////////////
- //
- // Commands database and parsing methods
- //
-////////////////////////////////////////////////////////////////////////
-CommandInfo const g_commands[] = {
-    { CMD_BURN           , "burn"  ,false , 0, CI_IMG_AND_DEV , ""},
-    { CMD_BURN_BLOCK     , "bb"    ,true  , 0, CI_IMG_AND_DEV , ""},
-    { CMD_SET_GUIDS      , "sg"    ,true  , 1, CI_IMG_OR_DEV  , ""},
-    { CMD_SET_MFG_GUIDS  , "smg"    ,true  , 0, CI_IMG_ONLY  , ""},
-    { CMD_BURN_VPD       , "set_vpd"    ,true  , 1, CI_IMG_ONLY  , ""},
-
-
-    { CMD_SET_VSD        , "sv"    ,true  , 0, CI_IMG_OR_DEV  , ""},
-    { CMD_SET_KEY        , "set_key" ,true , 1, CI_DEV_ONLY  , ""},
-    { CMD_QUERY_FORCE    , "qf"    ,true  , 0, CI_IMG_OR_DEV  , ""},
-    { CMD_QUERY          , "query" ,false , 1, CI_IMG_OR_DEV  , ""},
-    { CMD_QUERY_ROM      , "qrom"  ,true  , 0, CI_IMG_ONLY    , ""},
-    { CMD_VERIFY         , "verify",false , 1, CI_IMG_OR_DEV  , ""},
-    { CMD_READ_WORD      , "rw"    ,true  , 1, CI_DEV_ONLY    , ""},
-    { CMD_READ_BLOCK     , "rb"    ,true  , 3, CI_IMG_OR_DEV  , ""},
-    { CMD_WRITE_WORD     , "ww"    ,true  , 2, CI_DEV_ONLY    , ""},
-    { CMD_WRITE_WORD_NE  , "wwne"  ,true  , 2, CI_DEV_ONLY    , ""},
-    { CMD_WRITE_BLOCK    , "wb"    ,true  , 2, CI_DEV_ONLY    , ""},
-    { CMD_WRITE_BLOCK_NE , "wbne"  ,true  ,-1, CI_DEV_ONLY    , ""},
-    { CMD_ERASE_SECT     , "erase" ,false , 1, CI_DEV_ONLY    , ""},
-    { CMD_DUMP_CONF      , "dc"    ,true  , 1, CI_IMG_OR_DEV  , ""},
-    { CMD_DUMP_HASH      , "dh"    ,true  , 1, CI_IMG_OR_DEV  , ""},
-    { CMD_DUMP_JSON      , "dj"    ,true  , 1, CI_IMG_OR_DEV  , ""},
-    { CMD_READ_IMAGE     , "ri"    ,true  , 1, CI_DEV_ONLY    , ""},
-    { CMD_CLEAR_SEM      , "clear_semaphore"   ,true  , 0 , CI_DEV_ONLY    , ""},
-    { CMD_SWRESET        , "swreset" ,true , 0, CI_DEV_ONLY    , ""},
-    { CMD_HW_INFO        , "hw"      ,false  , 2, CI_DEV_ONLY    , ""},
-    { CMD_BURN_ROM       , "brom"    ,true  , 1, CI_DEV_ONLY    , ""},
-    { CMD_REMOVE_ROM     , "drom"    ,true  , 0, CI_DEV_ONLY    , ""},
-    { CMD_READ_ROM       , "rrom"    ,true  , 1, CI_DEV_ONLY   , ""},
-    { CMD_HW_ACCESS      , "hw_access" , true , 2, CI_DEV_ONLY   , ""},
-    { CMD_DUMP_TRACER_HASH , "dth"    ,true  , 1, CI_IMG_OR_DEV  , ""},
-
-
-};
-
 struct GPH {
     u_int32_t type;
     u_int32_t size;
@@ -438,12 +384,6 @@ struct BOARD_ID {
     char      bid[BOARD_ID_BID_LEN];
 };
 
-/*
-int  const VSD_LEN  = 208;
-int  const PSID_LEN = 16;
-int  const PRODUCT_VER_LEN = 16;
-*/
-
 #define PROFILES_LIST_SECT "Profiles List section"
 #define TLV_FORMAT_SECT    "TLVs format section"
 #define TRACER_HASH_SECT   "Tracer Hash section"
@@ -454,7 +394,6 @@ void report_callback(f_prog_func_str func_str, const char *format, ...);
 void report_err(char err_buff[MAX_ERR_STR_LEN], const char *format, ...);
 void report_warn(const char *format, ...);
 void report_repair_msg(const char* common_msg);
-const CommandInfo* GetCommandInfo(CommandType cmd);
 
 
 
@@ -463,12 +402,13 @@ const CommandInfo* GetCommandInfo(CommandType cmd);
 //                  BASE CLASSES
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
-class MLXFWOP_API ErrMsg {
+class MLXFWOP_API FlintErrMsg {
 public:
-    ErrMsg() : _err(0)       {}
-    ~ErrMsg()                { err_clear();}
+    FlintErrMsg() : _err(0), _errCode(0){}
+    ~FlintErrMsg()                { err_clear();}
     const char *err() const  { return _err;}
     void err_clear();
+    int getErrorCode() {return _errCode;}
 
 protected:
 
@@ -496,6 +436,12 @@ protected:
 #endif
     ;
 
+    bool errmsg(int errorCode, const char *format, ...)
+#ifdef __GNUC__
+    __attribute__ ((format (printf, 3, 4)))
+#endif
+    ;
+
     // this is abit ugly as there are no checks on the normalFmt string
     bool errmsgAdv(bool showAdv, const char *normalFmt, const char *AdvFmt, ...)
 #ifdef __GNUC__
@@ -503,9 +449,16 @@ protected:
 #endif
     ;
 
+    int errmsgWCode(int errorCode, const char *format, ...)
+#ifdef __GNUC__
+    __attribute__ ((format (printf, 3, 4)))
+#endif
+    ;
+
 private:
 
     char       *_err;
+    int         _errCode;
 };
 
 
