@@ -28,6 +28,7 @@
  * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
+ *
  */
 
 /********************************************************
@@ -749,6 +750,11 @@ int mib_open(const char *name, mfile *mf, int mad_init)
     return 0;
 end:
     if (ivm) {
+#ifndef __WIN__
+        if (ivm->dl_handle) {
+            dlclose(ivm->dl_handle);
+        }
+#endif
         free(ivm);
     }
     if (nbuf) {
@@ -849,38 +855,45 @@ mib_write4(mfile *mf, u_int32_t memory_address, u_int32_t _data)
     return 4;
 }
 
-MTCR_API int mib_readblock(mfile *mf, unsigned int offset, u_int32_t *data, int length)
+enum {
+    BLOCKOP_READ,
+    BLOCKOP_WRITE
+};
+
+int mib_block_op(mfile *mf, unsigned int offset, u_int32_t *data, int length, int op)
 {
     if (!mf || !mf->ctx || !data) {
         IBERROR(("cr access read failed. Null Param."));
         return -1;
     }
     ibvs_mad* h = (ibvs_mad*)(mf->ctx);
-
+    int method = IB_MAD_METHOD_GET;
+    if (op == BLOCKOP_WRITE) {
+        method = IB_MAD_METHOD_SET;
+    }
     CHECK_ALIGN(length);
-
-    if (ibvsmad_craccess_rw(h, offset, IB_MAD_METHOD_GET, (length / 4), data) == ~0ull) {
-        IBERROR(("cr access read to %s failed", h->portid2str(&h->portid)));
-        return -1;
+    int chunk_size = mib_get_chunk_size(mf);
+    int t_offset = 0;
+    while (t_offset < length) {
+        int left_size = length - t_offset;
+        int to_op = left_size > chunk_size ? chunk_size : left_size;
+        if (ibvsmad_craccess_rw(h, offset + t_offset, method, (to_op / 4), data + t_offset/4) == ~0ull) {
+            IBERROR(("cr access %s to %s failed", op == BLOCKOP_READ ? "read" : "write",
+                    h->portid2str(&h->portid)));
+            return -1;
+        }
+        t_offset += chunk_size;
     }
     return length;
+}
 
+MTCR_API int mib_readblock(mfile *mf, unsigned int offset, u_int32_t *data, int length)
+{
+    return mib_block_op(mf, offset, data, length, BLOCKOP_READ);
 }
 MTCR_API int mib_writeblock(mfile *mf, unsigned int offset, u_int32_t *data, int length)
 {
-    if (!mf || !mf->ctx || !data) {
-        IBERROR(("cr access write failed. Null Param."));
-        return -1;
-    }
-    ibvs_mad* h = (ibvs_mad*)(mf->ctx);
-
-    CHECK_ALIGN(length);
-
-    if (ibvsmad_craccess_rw(h, offset, IB_MAD_METHOD_SET, (length / 4), data) == ~0ull) {
-        IBERROR(("cr access read to %s failed", h->portid2str(&h->portid)));
-        return -1;
-    }
-    return length;
+    return mib_block_op(mf, offset, data, length, BLOCKOP_WRITE);
 }
 
 int is_node_managed(ibvs_mad* h)
